@@ -72,10 +72,6 @@ typedef char ICHAR;
 
 #define EXPAND_SPARE 24
 
-/* On macintosh, '\n' may get translated to 0xD instead of 0xA */
-#define CR '\015'
-#define LF '\012'
-
 typedef struct binding {
   struct prefix *prefix;
   struct binding *nextTagBinding;
@@ -91,11 +87,17 @@ typedef struct prefix {
   BINDING *binding;
 } PREFIX;
 
+typedef struct {
+  const XML_Char *str;
+  const XML_Char *localPart;
+  int uriLen;
+} TAG_NAME;
+
 typedef struct tag {
   struct tag *parent;
   const char *rawName;
   int rawNameLength;
-  const XML_Char *name;
+  TAG_NAME name;
   char *buf;
   char *bufEnd;
   BINDING *bindings;
@@ -196,7 +198,7 @@ doContent(XML_Parser parser, int startTagLevel, const ENCODING *enc,
 static enum XML_Error
 doCdataSection(XML_Parser parser, const ENCODING *, const char **startPtr, const char *end, const char **nextPtr);
 static enum XML_Error storeAtts(XML_Parser parser, const ENCODING *, const char *s,
-				const XML_Char **tagNamePtr, BINDING **bindingsPtr);
+				TAG_NAME *tagNamePtr, BINDING **bindingsPtr);
 static
 int addBinding(XML_Parser parser, PREFIX *prefix, const ATTRIBUTE_ID *attId, const XML_Char *uri, BINDING **bindingsPtr);
 static int
@@ -273,6 +275,8 @@ typedef struct {
   XML_DefaultHandler defaultHandler;
   XML_UnparsedEntityDeclHandler unparsedEntityDeclHandler;
   XML_NotationDeclHandler notationDeclHandler;
+  XML_StartNamespaceDeclHandler startNamespaceDeclHandler;
+  XML_EndNamespaceDeclHandler endNamespaceDeclHandler;
   XML_ExternalEntityRefHandler externalEntityRefHandler;
   void *externalEntityRefHandlerArg;
   XML_UnknownEncodingHandler unknownEncodingHandler;
@@ -327,6 +331,8 @@ typedef struct {
 #define defaultHandler (((Parser *)parser)->defaultHandler)
 #define unparsedEntityDeclHandler (((Parser *)parser)->unparsedEntityDeclHandler)
 #define notationDeclHandler (((Parser *)parser)->notationDeclHandler)
+#define startNamespaceDeclHandler (((Parser *)parser)->startNamespaceDeclHandler)
+#define endNamespaceDeclHandler (((Parser *)parser)->endNamespaceDeclHandler)
 #define externalEntityRefHandler (((Parser *)parser)->externalEntityRefHandler)
 #define externalEntityRefHandlerArg (((Parser *)parser)->externalEntityRefHandlerArg)
 #define unknownEncodingHandler (((Parser *)parser)->unknownEncodingHandler)
@@ -405,6 +411,8 @@ XML_Parser XML_ParserCreate(const XML_Char *encodingName)
   defaultHandler = 0;
   unparsedEntityDeclHandler = 0;
   notationDeclHandler = 0;
+  startNamespaceDeclHandler = 0;
+  endNamespaceDeclHandler = 0;
   externalEntityRefHandler = 0;
   externalEntityRefHandlerArg = parser;
   unknownEncodingHandler = 0;
@@ -510,6 +518,8 @@ XML_Parser XML_ExternalEntityParserCreate(XML_Parser oldParser,
   XML_StartCdataSectionHandler oldStartCdataSectionHandler = startCdataSectionHandler;
   XML_EndCdataSectionHandler oldEndCdataSectionHandler = endCdataSectionHandler;
   XML_DefaultHandler oldDefaultHandler = defaultHandler;
+  XML_StartNamespaceDeclHandler oldStartNamespaceDeclHandler = startNamespaceDeclHandler;
+  XML_EndNamespaceDeclHandler oldEndNamespaceDeclHandler = endNamespaceDeclHandler;
   XML_ExternalEntityRefHandler oldExternalEntityRefHandler = externalEntityRefHandler;
   XML_UnknownEncodingHandler oldUnknownEncodingHandler = unknownEncodingHandler;
   void *oldUserData = userData;
@@ -530,6 +540,8 @@ XML_Parser XML_ExternalEntityParserCreate(XML_Parser oldParser,
   startCdataSectionHandler = oldStartCdataSectionHandler;
   endCdataSectionHandler = oldEndCdataSectionHandler;
   defaultHandler = oldDefaultHandler;
+  startNamespaceDeclHandler = oldStartNamespaceDeclHandler;
+  endNamespaceDeclHandler = oldEndNamespaceDeclHandler;
   externalEntityRefHandler = oldExternalEntityRefHandler;
   unknownEncodingHandler = oldUnknownEncodingHandler;
   userData = oldUserData;
@@ -683,6 +695,14 @@ void XML_SetNotationDeclHandler(XML_Parser parser,
   notationDeclHandler = handler;
 }
 
+void XML_SetNamespaceDeclHandler(XML_Parser parser,
+				 XML_StartNamespaceDeclHandler start,
+				 XML_EndNamespaceDeclHandler end)
+{
+  startNamespaceDeclHandler = start;
+  endNamespaceDeclHandler = end;
+}
+
 void XML_SetExternalEntityRefHandler(XML_Parser parser,
 				     XML_ExternalEntityRefHandler handler)
 {
@@ -817,13 +837,6 @@ enum XML_Error XML_GetErrorCode(XML_Parser parser)
   return errorCode;
 }
 
-long XML_GetCurrentByteIndex(XML_Parser parser)
-{
-  if (eventPtr)
-    return parseEndByteIndex - (parseEndPtr - eventPtr);
-  return -1;
-}
-
 long XML_GetCurrentByteLimit(XML_Parser parser)
 {
   if (eventPtr)
@@ -831,6 +844,13 @@ long XML_GetCurrentByteLimit(XML_Parser parser)
   return -1;
 }
 
+long XML_GetCurrentByteIndex(XML_Parser parser)
+{
+  if (eventPtr)
+    return parseEndByteIndex - (parseEndPtr - eventPtr);
+  return -1;
+}
+
 int XML_GetCurrentLineNumber(XML_Parser parser)
 {
   if (eventPtr) {
@@ -1024,7 +1044,7 @@ doContent(XML_Parser parser,
       }
       *eventEndPP = end;
       if (characterDataHandler) {
-	XML_Char c = XML_T(LF);
+	XML_Char c = 0xA;
 	characterDataHandler(handlerArg, &c, 1);
       }
       else if (defaultHandler)
@@ -1163,10 +1183,16 @@ doContent(XML_Parser parser,
 	tag->bindings = 0;
 	tag->parent = tagStack;
 	tagStack = tag;
+	tag->name.localPart = 0;
 	tag->rawName = s + enc->minBytesPerChar;
 	tag->rawNameLength = XmlNameLength(enc, tag->rawName);
 	if (nextPtr) {
-	  if (tag->rawNameLength > tag->bufEnd - tag->buf) {
+	  /* Need to guarantee that:
+	  ** tag->buf + ROUND_UP(tag->rawNameLength, sizeof(XML_Char))
+          **      <= tag->bufEnd - sizeof(XML_Char)
+	  **/
+	  if (tag->rawNameLength + (int)(sizeof(XML_Char) - 1)
+	      + (int)sizeof(XML_Char) > tag->bufEnd - tag->buf) {
 	    int bufSize = tag->rawNameLength * 4;
 	    bufSize = ROUND_UP(bufSize, sizeof(XML_Char));
 	    tag->buf = realloc(tag->buf, bufSize);
@@ -1190,7 +1216,7 @@ doContent(XML_Parser parser,
 	      toPtr = (XML_Char *)(tag->buf + ROUND_UP(tag->rawNameLength, sizeof(XML_Char)));
 	    else
 	      toPtr = (XML_Char *)tag->buf;
-	    tag->name = toPtr;
+	    tag->name.str = toPtr;
 	    XmlConvert(enc,
 		       &fromPtr, rawNameEnd,
 		       (ICHAR **)&toPtr, (ICHAR *)tag->bufEnd - 1);
@@ -1208,8 +1234,10 @@ doContent(XML_Parser parser,
 	  result = storeAtts(parser, enc, s, &(tag->name), &(tag->bindings));
 	  if (result)
 	    return result;
-	  startElementHandler(handlerArg, tag->name, (const XML_Char **)atts);
-	  for (i = 0; i < attsSize; i += 2) {
+	  startElementHandler(handlerArg, tag->name.str, (const XML_Char **)atts);
+	  for (i = 0;
+	       i < attsSize * (sizeof(ATTRIBUTE)/sizeof(XML_Char **));
+	       i += 2) {
 	    if (! ((XML_Char **) atts)[i])
 	      break;
 
@@ -1218,7 +1246,7 @@ doContent(XML_Parser parser,
 	  poolClear(&tempPool);
 	}
 	else {
-	  tag->name = 0;
+	  tag->name.str = 0;
 	  if (defaultHandler)
 	    reportDefault(parser, enc, s, next);
 	}
@@ -1236,10 +1264,10 @@ doContent(XML_Parser parser,
 	const char *rawName = s + enc->minBytesPerChar;
 	enum XML_Error result;
 	BINDING *bindings = 0;
-	const XML_Char *name = poolStoreString(&tempPool, enc, rawName,
-					       rawName
-					       + XmlNameLength(enc, rawName));
-	if (!name)
+	TAG_NAME name;
+	name.str = poolStoreString(&tempPool, enc, rawName,
+				   rawName + XmlNameLength(enc, rawName));
+	if (!name.str)
 	  return XML_ERROR_NO_MEMORY;
 	poolFinish(&tempPool);
 	result = storeAtts(parser, enc, s, &name, &bindings);
@@ -1248,8 +1276,10 @@ doContent(XML_Parser parser,
 	poolFinish(&tempPool);
 	if (startElementHandler) {
 	  int i;
-	  startElementHandler(handlerArg, name, (const XML_Char **)atts);
-	  for (i = 0; i < attsSize; i += 2) {
+	  startElementHandler(handlerArg, name.str, (const XML_Char **)atts);
+	  for (i = 0;
+	       i < attsSize * (sizeof(ATTRIBUTE)/sizeof(XML_Char **));
+	       i += 2) {
 	    if (! ((XML_Char **) atts)[i])
 	      break;
 
@@ -1259,11 +1289,13 @@ doContent(XML_Parser parser,
 	if (endElementHandler) {
 	  if (startElementHandler)
 	    *eventPP = *eventEndPP;
-	  endElementHandler(handlerArg, name);
+	  endElementHandler(handlerArg, name.str);
 	}
 	poolClear(&tempPool);
 	while (bindings) {
 	  BINDING *b = bindings;
+	  if (endNamespaceDeclHandler)
+	    endNamespaceDeclHandler(handlerArg, b->prefix->name);
 	  bindings = bindings->nextTagBinding;
 	  b->nextTagBinding = freeBindingList;
 	  freeBindingList = b;
@@ -1293,12 +1325,21 @@ doContent(XML_Parser parser,
 	  return XML_ERROR_TAG_MISMATCH;
 	}
 	--tagLevel;
-	if (endElementHandler && tag->name)
-	  endElementHandler(handlerArg, tag->name);
+	if (endElementHandler && tag->name.str) {
+	  if (tag->name.localPart) {
+	    XML_Char *to = (XML_Char *)tag->name.str + tag->name.uriLen;
+	    const XML_Char *from = tag->name.localPart;
+	    while ((*to++ = *from++) != 0)
+	      ;
+	  }
+	  endElementHandler(handlerArg, tag->name.str);
+	}
 	else if (defaultHandler)
 	  reportDefault(parser, enc, s, next);
 	while (tag->bindings) {
 	  BINDING *b = tag->bindings;
+	  if (endNamespaceDeclHandler)
+	    endNamespaceDeclHandler(handlerArg, b->prefix->name);
 	  tag->bindings = tag->bindings->nextTagBinding;
 	  b->nextTagBinding = freeBindingList;
 	  freeBindingList = b;
@@ -1325,7 +1366,7 @@ doContent(XML_Parser parser,
       return XML_ERROR_MISPLACED_XML_PI;
     case XML_TOK_DATA_NEWLINE:
       if (characterDataHandler) {
-	XML_Char c = XML_T(LF);
+	XML_Char c = 0xA;
 	characterDataHandler(handlerArg, &c, 1);
       }
       else if (defaultHandler)
@@ -1430,7 +1471,7 @@ doContent(XML_Parser parser,
 otherwise just check the attributes for well-formedness. */
 
 static enum XML_Error storeAtts(XML_Parser parser, const ENCODING *enc,
-				const char *s, const XML_Char **tagNamePtr,
+				const char *s, TAG_NAME *tagNamePtr,
 				BINDING **bindingsPtr)
 {
   ELEMENT_TYPE *elementType = 0;
@@ -1444,12 +1485,12 @@ static enum XML_Error storeAtts(XML_Parser parser, const ENCODING *enc,
   const XML_Char *localPart;
 
   if (tagNamePtr) {
-    elementType = (ELEMENT_TYPE *)lookup(&dtd.elementTypes, *tagNamePtr, 0);
+    elementType = (ELEMENT_TYPE *)lookup(&dtd.elementTypes, tagNamePtr->str, 0);
     if (!elementType) {
-      *tagNamePtr = poolCopyString(&dtd.pool, *tagNamePtr);
-      if (!*tagNamePtr)
+      tagNamePtr->str = poolCopyString(&dtd.pool, tagNamePtr->str);
+      if (!tagNamePtr->str)
 	return XML_ERROR_NO_MEMORY;
-      elementType = (ELEMENT_TYPE *)lookup(&dtd.elementTypes, *tagNamePtr, sizeof(ELEMENT_TYPE));
+      elementType = (ELEMENT_TYPE *)lookup(&dtd.elementTypes, tagNamePtr->str, sizeof(ELEMENT_TYPE));
       if (!elementType)
         return XML_ERROR_NO_MEMORY;
       if (ns && !setElementTypePrefix(parser, elementType))
@@ -1596,16 +1637,18 @@ static enum XML_Error storeAtts(XML_Parser parser, const ENCODING *enc,
     binding = elementType->prefix->binding;
     if (!binding)
       return XML_ERROR_NONE;
-    localPart = *tagNamePtr;
+    localPart = tagNamePtr->str;
     while (*localPart++ != XML_T(':'))
       ;
   }
   else if (dtd.defaultPrefix.binding) {
     binding = dtd.defaultPrefix.binding;
-    localPart = *tagNamePtr;
+    localPart = tagNamePtr->str;
   }
   else
     return XML_ERROR_NONE;
+  tagNamePtr->localPart = localPart;
+  tagNamePtr->uriLen = binding->uriLen;
   i = binding->uriLen;
   do {
     if (i == binding->uriAlloc) {
@@ -1615,7 +1658,7 @@ static enum XML_Error storeAtts(XML_Parser parser, const ENCODING *enc,
     }
     binding->uri[i++] = *localPart;
   } while (*localPart++);
-  *tagNamePtr = binding->uri;
+  tagNamePtr->str = binding->uri;
   return XML_ERROR_NONE;
 }
 
@@ -1655,10 +1698,16 @@ int addBinding(XML_Parser parser, PREFIX *prefix, const ATTRIBUTE_ID *attId, con
     b->uri[len - 1] = namespaceSeparator;
   b->prefix = prefix;
   b->attId = attId;
-  b->prevPrefixBinding = b->prefix->binding;
-  b->prefix->binding = *uri == XML_T('\0') ? 0 : b;
+  b->prevPrefixBinding = prefix->binding;
+  if (*uri == XML_T('\0') && prefix == &dtd.defaultPrefix)
+    prefix->binding = 0;
+  else
+    prefix->binding = b;
   b->nextTagBinding = *bindingsPtr;
   *bindingsPtr = b;
+  if (startNamespaceDeclHandler)
+    startNamespaceDeclHandler(handlerArg, prefix->name,
+			      prefix->binding ? uri : 0);
   return 1;
 }
 
@@ -1722,7 +1771,7 @@ enum XML_Error doCdataSection(XML_Parser parser,
       return XML_ERROR_NONE;
     case XML_TOK_DATA_NEWLINE:
       if (characterDataHandler) {
-	XML_Char c = XML_T(LF);
+	XML_Char c = 0xA;
 	characterDataHandler(handlerArg, &c, 1);
       }
       else if (defaultHandler)
@@ -2284,7 +2333,7 @@ storeAttributeValue(XML_Parser parser, const ENCODING *enc, int isCdata,
   enum XML_Error result = appendAttributeValue(parser, enc, isCdata, ptr, end, pool);
   if (result)
     return result;
-  if (!isCdata && poolLength(pool) && poolLastChar(pool) == XML_T(' '))
+  if (!isCdata && poolLength(pool) && poolLastChar(pool) == 0x20)
     poolChop(pool);
   if (!poolAppendChar(pool, XML_T('\0')))
     return XML_ERROR_NO_MEMORY;
@@ -2323,7 +2372,7 @@ appendAttributeValue(XML_Parser parser, const ENCODING *enc, int isCdata,
 	}
 	if (!isCdata
 	    && n == 0x20 /* space */
-	    && (poolLength(pool) == 0 || poolLastChar(pool) == XML_T(' ')))
+	    && (poolLength(pool) == 0 || poolLastChar(pool) == 0x20))
 	  break;
 	n = XmlEncode(n, (ICHAR *)buf);
 	if (!n) {
@@ -2347,9 +2396,9 @@ appendAttributeValue(XML_Parser parser, const ENCODING *enc, int isCdata,
       /* fall through */
     case XML_TOK_ATTRIBUTE_VALUE_S:
     case XML_TOK_DATA_NEWLINE:
-      if (!isCdata && (poolLength(pool) == 0 || poolLastChar(pool) == XML_T(' ')))
+      if (!isCdata && (poolLength(pool) == 0 || poolLastChar(pool) == 0x20))
 	break;
-      if (!poolAppendChar(pool, XML_T(' ')))
+      if (!poolAppendChar(pool, 0x20))
 	return XML_ERROR_NO_MEMORY;
       break;
     case XML_TOK_ENTITY_REF:
@@ -2448,7 +2497,7 @@ enum XML_Error storeEntityValue(XML_Parser parser,
     case XML_TOK_DATA_NEWLINE:
       if (pool->end == pool->ptr && !poolGrow(pool))
 	return XML_ERROR_NO_MEMORY;
-      *(pool->ptr)++ = XML_T(LF);
+      *(pool->ptr)++ = 0xA;
       break;
     case XML_TOK_CHAR_REF:
       {
@@ -2492,14 +2541,14 @@ normalizeLines(XML_Char *s)
   for (;; s++) {
     if (*s == XML_T('\0'))
       return;
-    if (*s == XML_T(CR))
+    if (*s == 0xD)
       break;
   }
   p = s;
   do {
-    if (*s == XML_T(CR)) {
-      *p++ = XML_T(LF);
-      if (*++s == XML_T(LF))
+    if (*s == 0xD) {
+      *p++ = 0xA;
+      if (*++s == 0xA)
         s++;
     }
     else
@@ -2826,17 +2875,17 @@ void normalizePublicId(XML_Char *publicId)
   XML_Char *s;
   for (s = publicId; *s; s++) {
     switch (*s) {
-    case XML_T(' '):
-    case XML_T(CR):
-    case XML_T(LF):
-      if (p != publicId && p[-1] != XML_T(' '))
-	*p++ = XML_T(' ');
+    case 0x20:
+    case 0xD:
+    case 0xA:
+      if (p != publicId && p[-1] != 0x20)
+	*p++ = 0x20;
       break;
     default:
       *p++ = *s;
     }
   }
-  if (p != publicId && p[-1] == XML_T(' '))
+  if (p != publicId && p[-1] == 0x20)
     --p;
   *p = XML_T('\0');
 }
